@@ -20,13 +20,12 @@ from bec_lib import (
     messages,
 )
 from bec_lib.connector import ConnectorBase
+from device_server.devices.config_update_handler import ConfigUpdateHandler
+from device_server.devices.device_serializer import get_device_info
 from ophyd.ophydobj import OphydObject
 from ophyd.signal import EpicsSignalBase
 from typeguard import typechecked
 
-from device_server.devices.config_update_handler import ConfigUpdateHandler
-from device_server.devices.device_serializer import get_device_info
-
 try:
     from bec_plugins import devices as plugin_devices
 except ImportError:
@@ -91,6 +90,7 @@ class DeviceManagerDS(DeviceManagerBase):
         self._device_instructions_connector = None
         self._config_update_handler_cls = config_update_handler
         self.config_update_handler = None
+        self.failed_devices = []
 
     def initialize(self, bootstrap_server) -> None:
         self.config_update_handler = (
@@ -101,14 +101,7 @@ class DeviceManagerDS(DeviceManagerBase):
         super().initialize(bootstrap_server)
 
     def _reload_action(self) -> None:
-        for dev, obj in self.devices.items():
-            try:
-                obj.obj.destroy()
-            except Exception:
-                logger.warning(f"Failed to destroy {obj.obj.name}")
-                raise RuntimeError
-        self.devices.flush()
-        self._get_config()
+        pass
 
     @staticmethod
     def _get_device_class(dev_type: str) -> type:
@@ -138,16 +131,90 @@ class DeviceManagerDS(DeviceManagerBase):
         return getattr(module, class_name)
 
     def _load_session(self, *_args, **_kwargs):
-        if self._is_config_valid():
+        delayed_init = []
+        if not self._is_config_valid():
+            self._reset_config()
+            return
+
+        try:
+            self.failed_devices = {}
             for dev in self._session["devices"]:
                 name = dev.get("name")
                 enabled = dev.get("enabled")
                 logger.info(f"Adding device {name}: {'ENABLED' if enabled else 'DISABLED'}")
                 try:
+                    dev_cls = self._get_device_class(dev.get("deviceClass"))
+                    if issubclass(dev_cls, opd.DeviceProxy):
+                        delayed_init.append(dev)
+                        continue
                     self.initialize_device(dev)
-                except Exception as exc:
-                    content = traceback.format_exc()
-                    logger.error(f"Failed to initialize device: {dev}: {content}.")
+                except ConnectionError:
+                    msg = traceback.format_exc()
+                    self.failed_devices[name] = msg
+
+            for dev in delayed_init:
+                try:
+                    name = dev.get("name")
+                    self.initialize_delayed_devices(dev)
+                except (ConnectionError, DeviceConfigError):
+                    msg = traceback.format_exc()
+                    self.failed_devices[name] = msg
+            self.config_update_handler.handle_failed_device_inits()
+        except Exception as exc:
+            content = traceback.format_exc()
+            logger.error(
+                f"Failed to initialize device: {dev}: {content}. The config will be reset."
+            )
+            self._reset_config()
+            raise DeviceConfigError(
+                f"Failed to initialize device: {dev}: {content}. The config will be reset."
+            ) from exc
+
+    def initialize_delayed_devices(self, dev: dict) -> None:
+        """Initialize delayed device after all other devices have been initialized."""
+        name = dev.get("name")
+        enabled = dev.get("enabled")
+        logger.info(f"Adding device {name}: {'ENABLED' if enabled else 'DISABLED'}")
+        self.initialize_device(dev)
+
+        obj_lookup = self.devices.get(name).obj.lookup
+        for key in obj_lookup.keys():
+            signal_name = obj_lookup[key].get("signal_name")
+            if key not in self.devices:
+                raise DeviceConfigError(
+                    f"Failed to init DeviceProxy {name}, no device {key} found in device manager."
+                )
+            dev_obj = self.devices[key].obj
+            registered_proxies = dev_obj.registered_proxies
+            if not hasattr(dev_obj, signal_name):
+                raise DeviceConfigError(
+                    f"Failed to init DeviceProxy {name}, no signal {signal_name} found for device {key}."
+                )
+            if key not in registered_proxies:
+                self.devices[key].obj._registered_proxies.update({name: signal_name})
+                continue
+            if key in registered_proxies and signal_name not in registered_proxies[key]:
+                self.devices[key].obj._registered_proxies.update({name: signal_name})
+                continue
+            if key in registered_proxies.keys() and signal_name in registered_proxies[key]:
+                raise RuntimeError(
+                    f"Failed to init DeviceProxy {name}, device {key} already has a registered DeviceProxy for {signal_name}. Only one DeviceProxy can be active per signal."
+                )
+
+    def _reset_config(self):
+        current_config = self._session["devices"]
+        if current_config:
+            # store the current config in the history
+            current_config_msg = messages.AvailableResourceMessage(
+                resource=current_config, metadata={"removed_at": time.time()}
+            )
+            self.producer.lpush(
+                MessageEndpoints.device_config_history(), current_config_msg, max_size=50
+            )
+        msg = messages.AvailableResourceMessage(resource=[])
+        self.producer.set(MessageEndpoints.device_config(), msg)
+        reload_msg = messages.DeviceConfigMessage(action="reload", config={})
+        self.producer.send(MessageEndpoints.device_config_update(), reload_msg)
 
     @staticmethod
     def update_config(obj: OphydObject, config: dict) -> None:
@@ -158,8 +225,8 @@ class DeviceManagerDS(DeviceManagerBase):
             config (dict): Config dictionary
 
         """
-        if hasattr(obj, "update_config"):
-            obj.update_config(config)
+        if hasattr(obj, "_update_device_config"):
+            obj._update_device_config(config)
         else:
             for config_key, config_value in config.items():
                 # first handle the ophyd exceptions...
@@ -266,18 +333,21 @@ class DeviceManagerDS(DeviceManagerBase):
             return opaas_obj
 
         # update device buffer for enabled devices
-        try:
-            self.initialize_enabled_device(opaas_obj)
+        # try:
+        self.initialize_enabled_device(opaas_obj)
         # pylint:disable=broad-except
-        except Exception:
-            error_traceback = traceback.format_exc()
-            logger.error(
-                f"{error_traceback}. Failed to stage {opaas_obj.name}. The device will be disabled."
-            )
-            opaas_obj.enabled = False
+        # except Exception:
+        #     error_traceback = traceback.format_exc()
+        #     logger.error(
+        #         f"{error_traceback}. Failed to stage {opaas_obj.name}. The device will be disabled."
+        #     )
+        #     opaas_obj.enabled = False
 
         obj = opaas_obj.obj
         # add subscriptions
+        if not hasattr(obj, "event_types"):
+            return opaas_obj
+
         if "readback" in obj.event_types:
             obj.subscribe(self._obj_callback_readback, run=opaas_obj.enabled)
         elif "value" in obj.event_types:
@@ -318,23 +388,27 @@ class DeviceManagerDS(DeviceManagerBase):
     @staticmethod
     def connect_device(obj, wait_for_all=False):
         """establish a connection to a device"""
-        if obj.connected:
-            return
-        if hasattr(obj, "controller"):
-            obj.controller.on()
-            return
-        if hasattr(obj, "wait_for_connection"):
-            try:
-                obj.wait_for_connection(all_signals=wait_for_all, timeout=10)
-            except TypeError:
-                obj.wait_for_connection(timeout=10)
-            return
-
-        logger.error(
-            f"Device {obj.name} does not implement the socket controller interface nor"
-            " wait_for_connection and cannot be turned on."
-        )
-        raise ConnectionError(f"Failed to establish a connection to device {obj.name}")
+        try:
+            if obj.connected:
+                return
+            if hasattr(obj, "controller"):
+                obj.controller.on()
+                return
+            if hasattr(obj, "wait_for_connection"):
+                try:
+                    obj.wait_for_connection(all_signals=wait_for_all, timeout=10)
+                except TypeError:
+                    obj.wait_for_connection(timeout=10)
+                return
+            logger.error(
+                f"Device {obj.name} does not implement the socket controller interface nor"
+                " wait_for_connection and cannot be turned on."
+            )
+            raise ConnectionError(f"Failed to establish a connection to device {obj.name}")
+        except Exception:
+            error_traceback = traceback.format_exc()
+            logger.error(f"{error_traceback}. Failed to connect to {obj.name}.")
+            raise ConnectionError(f"Failed to establish a connection to device {obj.name}")
 
     def publish_device_info(self, obj: OphydObject, pipe=None) -> None:
         """
